"""#!/usr/bin/env python
# -*- coding: utf-8 -*-

from shimehari import Request, Response, ApplicationController

class %sController(ApplicationController):
    def __init__(self, name):
        ApplicationController.__init__(self,name)

    #Create your controller here.

    def index(self, *args, **kwargs):
        pass

    def show(self, *args, **kwargs):
        pass

    def edit(self, *args, **kwargs):
        pass

    def new(self, *args, **kwargs):
        pass

    def create(self, *args, **kwargs):
        pass

    def update(self, *args, **kwargs):
        pass

    def destroy(self, *args, **kwargs):
        pass

"""
